@@ -1,88 +1,90 @@
 package org.apache.velocity.anakia;
 
 /*
- * Copyright 2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.    
  */
 
 import java.io.BufferedWriter;
 import java.io.File;
 import java.io.FileOutputStream;
+import java.io.IOException;
 import java.io.OutputStreamWriter;
 import java.io.Writer;
-
+import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.StringTokenizer;
 
+import org.apache.commons.collections.ExtendedProperties;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.DirectoryScanner;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.taskdefs.MatchingTask;
-
-import org.xml.sax.SAXParseException;
-
-import org.jdom.Document;
-import org.jdom.JDOMException;
-import org.jdom.input.SAXBuilder;
-
 import org.apache.velocity.Template;
+import org.apache.velocity.VelocityContext;
 import org.apache.velocity.app.VelocityEngine;
 import org.apache.velocity.runtime.RuntimeConstants;
 import org.apache.velocity.util.StringUtils;
 
-import org.apache.velocity.VelocityContext;
+import org.jdom.Document;
+import org.jdom.JDOMException;
+import org.jdom.input.SAXBuilder;
+import org.jdom.output.Format;
+import org.xml.sax.SAXParseException;
 
 /**
- * The purpose of this Ant Task is to allow you to use 
+ * The purpose of this Ant Task is to allow you to use
  * Velocity as an XML transformation tool like XSLT is.
- * So, instead of using XSLT, you will be able to use this 
+ * So, instead of using XSLT, you will be able to use this
  * class instead to do your transformations. It works very
  * similar in concept to Ant's &lt;style&gt; task.
  * <p>
  * You can find more documentation about this class on the
- * Velocity 
- * <a href="http://jakarta.apache.org/velocity/anakia.html">Website</a>.
- *   
+ * Velocity
+ * <a href="http://velocity.apache.org/engine/devel/docs/anakia.html">Website</a>.
+ *
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
  * @author <a href="mailto:szegedia@freemail.hu">Attila Szegedi</a>
- * @version $Id: AnakiaTask.java,v 1.34.4.1 2004/03/03 23:22:04 geirm Exp $
+ * @version $Id: AnakiaTask.java 501574 2007-01-30 21:32:26Z henning $
  */
 public class AnakiaTask extends MatchingTask
 {
     /** <code>{@link SAXBuilder}</code> instance to use */
-    private SAXBuilder builder;
+    SAXBuilder builder;
 
     /** the destination directory */
     private File destDir = null;
-    
+
     /** the base directory */
-    private File baseDir = null;
+    File baseDir = null;
 
     /** the style= attribute */
     private String style = null;
-    
-    /** the File to the style file */
-    private File styleFile = null;
-    
+
     /** last modified of the style sheet */
     private long styleSheetLastModified = 0;
 
     /** the projectFile= attribute */
     private String projectAttribute = null;
-    
+
     /** the File for the project.xml file */
     private File projectFile = null;
-    
+
     /** last modified of the project file if it exists */
     private long projectFileLastModified = 0;
 
@@ -101,6 +103,9 @@ public class AnakiaTask extends MatchingTask
     /** the VelocityEngine instance to use */
     private VelocityEngine ve = new VelocityEngine();
 
+    /** the Velocity subcontexts */
+    private List contexts = new LinkedList();
+
     /**
      * Constructor creates the SAXBuilder.
      */
@@ -112,40 +117,44 @@ public class AnakiaTask extends MatchingTask
 
     /**
      * Set the base directory.
+     * @param dir
      */
     public void setBasedir(File dir)
     {
         baseDir = dir;
     }
-    
+
     /**
      * Set the destination directory into which the VSL result
      * files should be copied to
-     * @param dirName the name of the destination directory
+     * @param dir the name of the destination directory
      */
     public void setDestdir(File dir)
     {
         destDir = dir;
     }
-    
+
     /**
      * Allow people to set the default output file extension
+     * @param extension
      */
     public void setExtension(String extension)
     {
         this.extension = extension;
     }
-    
+
     /**
      * Allow people to set the path to the .vsl file
+     * @param style
      */
     public void setStyle(String style)
     {
         this.style = style;
     }
-    
+
     /**
      * Allow people to set the path to the project.xml file
+     * @param projectAttribute
      */
     public void setProjectFile(String projectAttribute)
     {
@@ -156,19 +165,20 @@ public class AnakiaTask extends MatchingTask
      * Set the path to the templates.
      * The way it works is this:
      * If you have a Velocity.properties file defined, this method
-     * will <strong>override</strong> whatever is set in the 
+     * will <strong>override</strong> whatever is set in the
      * Velocity.properties file. This allows one to not have to define
      * a Velocity.properties file, therefore using Velocity's defaults
      * only.
+     * @param templatePath
      */
-    
+
     public void setTemplatePath(File templatePath)
      {
-         try 
+         try
          {
              this.templatePath = templatePath.getCanonicalPath();
-         } 
-         catch (java.io.IOException ioe) 
+         }
+         catch (java.io.IOException ioe)
          {
              throw new BuildException(ioe);
          }
@@ -177,9 +187,10 @@ public class AnakiaTask extends MatchingTask
     /**
      * Allow people to set the path to the velocity.properties file
      * This file is found relative to the path where the JVM was run.
-     * For example, if build.sh was executed in the ./build directory, 
+     * For example, if build.sh was executed in the ./build directory,
      * then the path would be relative to this directory.
      * This is optional based on the setting of setTemplatePath().
+     * @param velocityPropertiesFile
      */
     public void setVelocityPropertiesFile(File velocityPropertiesFile)
     {
@@ -188,10 +199,11 @@ public class AnakiaTask extends MatchingTask
 
     /**
      * Turn on/off last modified checking. by default, it is on.
+     * @param lastmod
      */
     public void setLastModifiedCheck(String lastmod)
     {
-        if (lastmod.equalsIgnoreCase("false") || lastmod.equalsIgnoreCase("no") 
+        if (lastmod.equalsIgnoreCase("false") || lastmod.equalsIgnoreCase("no")
                 || lastmod.equalsIgnoreCase("off"))
         {
             this.lastModifiedCheck = false;
@@ -200,12 +212,12 @@ public class AnakiaTask extends MatchingTask
 
     /**
      * Main body of the application
+     * @throws BuildException
      */
     public void execute () throws BuildException
     {
         DirectoryScanner scanner;
         String[]         list;
-        String[]         dirs;
 
         if (baseDir == null)
         {
@@ -216,7 +228,7 @@ public class AnakiaTask extends MatchingTask
             String msg = "destdir attribute must be set!";
             throw new BuildException(msg);
         }
-        if (style == null) 
+        if (style == null)
         {
             throw new BuildException("style attribute must be set!");
         }
@@ -227,13 +239,13 @@ public class AnakiaTask extends MatchingTask
         }
 
         /*
-         * If the props file doesn't exist AND a templatePath hasn't 
+         * If the props file doesn't exist AND a templatePath hasn't
          * been defined, then throw the exception.
          */
         if ( !velocityPropertiesFile.exists() && templatePath == null )
         {
-            throw new BuildException ("No template path and could not " + 
-                "locate velocity.properties file: " + 
+            throw new BuildException ("No template path and could not " +
+                "locate velocity.properties file: " +
                 velocityPropertiesFile.getAbsolutePath());
         }
 
@@ -249,7 +261,7 @@ public class AnakiaTask extends MatchingTask
             }
             else
             {
-                log ("Project file is defined, but could not be located: " + 
+                log ("Project file is defined, but could not be located: " +
                     projectFile.getAbsolutePath(), Project.MSG_INFO );
                 projectFile = null;
             }
@@ -260,15 +272,20 @@ public class AnakiaTask extends MatchingTask
         {
             if ( velocityPropertiesFile.exists() )
             {
-                ve.init(velocityPropertiesFile.getAbsolutePath());
+                String file = velocityPropertiesFile.getAbsolutePath();
+                ExtendedProperties config = new ExtendedProperties(file);
+                ve.setExtendedProperties(config);
             }
-            else if (templatePath != null && templatePath.length() > 0)
+
+            // override the templatePath if it exists
+            if (templatePath != null && templatePath.length() > 0)
             {
                 ve.setProperty( RuntimeConstants.FILE_RESOURCE_LOADER_PATH,
                     templatePath);
-                ve.init();
             }
 
+            ve.init();
+
             // get the last modification of the VSL stylesheet
             styleSheetLastModified = ve.getTemplate( style ).getLastModified();
 
@@ -283,7 +300,7 @@ public class AnakiaTask extends MatchingTask
             log("Error: " + e.toString(), Project.MSG_INFO);
             throw new BuildException(e);
         }
-        
+
         // find the files/directories
         scanner = getDirectoryScanner(baseDir);
 
@@ -291,15 +308,15 @@ public class AnakiaTask extends MatchingTask
         list = scanner.getIncludedFiles();
         for (int i = 0;i < list.length; ++i)
         {
-            process( baseDir, list[i], destDir, projectDocument );
+            process(list[i], projectDocument );
         }
-    }    
-    
+
+    }
+
     /**
      * Process an XML file using Velocity
      */
-    private void process(File baseDir, String xmlFile, File destDir, 
-                         Document projectDocument)
+    private void process(String xmlFile, Document projectDocument)
         throws BuildException
     {
         File   outFile=null;
@@ -310,15 +327,16 @@ public class AnakiaTask extends MatchingTask
             // the current input file relative to the baseDir
             inFile = new File(baseDir,xmlFile);
             // the output file relative to basedir
-            outFile = new File(destDir, 
+            outFile = new File(destDir,
                             xmlFile.substring(0,
                             xmlFile.lastIndexOf('.')) + extension);
 
             // only process files that have changed
-            if (lastModifiedCheck == false || 
+            if (lastModifiedCheck == false ||
                     (inFile.lastModified() > outFile.lastModified() ||
                     styleSheetLastModified > outFile.lastModified() ||
-                    projectFileLastModified > outFile.lastModified()))
+                    projectFileLastModified > outFile.lastModified() ||
+                    userContextsModifed(outFile.lastModified())))
             {
                 ensureDirectoryFor( outFile );
 
@@ -336,15 +354,17 @@ public class AnakiaTask extends MatchingTask
                  *  we know it's a string...
                  */
                 String encoding = (String) ve.getProperty( RuntimeConstants.OUTPUT_ENCODING );
-                if (encoding == null || encoding.length() == 0 
+                if (encoding == null || encoding.length() == 0
                     || encoding.equals("8859-1") || encoding.equals("8859_1"))
                 {
                     encoding = "ISO-8859-1";
                 }
 
-                OutputWrapper ow = new OutputWrapper();
-                ow.setEncoding (encoding);
-                
+                Format f = Format.getRawFormat();
+                f.setEncoding(encoding);
+
+                OutputWrapper ow = new OutputWrapper(f);
+
                 context.put ("root", root.getRootElement());
                 context.put ("xmlout", ow );
                 context.put ("relativePath", getRelativePath(xmlFile));
@@ -353,18 +373,45 @@ public class AnakiaTask extends MatchingTask
                 context.put ("escape", new Escape() );
                 context.put ("date", new java.util.Date() );
 
-                // only put this into the context if it exists.
+                /**
+                 * only put this into the context if it exists.
+                 */
                 if (projectDocument != null)
                 {
                     context.put ("project", projectDocument.getRootElement());
                 }
-                
-                // Process the VSL template with the context and write out
-                // the result as the outFile.
+
+                /**
+                 *  Add the user subcontexts to the to context
+                 */
+                for (Iterator iter = contexts.iterator(); iter.hasNext();)
+                {
+                    Context subContext = (Context) iter.next();
+                    if (subContext == null)
+                    {
+                        throw new BuildException("Found an undefined SubContext!");
+                    }
+
+                    if (subContext.getContextDocument() == null)
+                    {
+                        throw new BuildException("Could not build a subContext for " + subContext.getName());
+                    }
+
+                    context.put(subContext.getName(), subContext
+                            .getContextDocument().getRootElement());
+                }
+
+                /**
+                 * Process the VSL template with the context and write out
+                 * the result as the outFile.
+                 */
                 writer = new BufferedWriter(new OutputStreamWriter(
                                             new FileOutputStream(outFile),
                                                 encoding));
-                // get the template to process
+
+                /**
+                 * get the template to process
+                 */
                 Template template = ve.getTemplate(style);
                 template.merge(context, writer);
 
@@ -373,7 +420,8 @@ public class AnakiaTask extends MatchingTask
         }
         catch (JDOMException e)
         {
-            if (outFile != null ) outFile.delete();
+            outFile.delete();
+
             if (e.getCause() != null)
             {
                 Throwable rootCause = e.getCause();
@@ -382,9 +430,9 @@ public class AnakiaTask extends MatchingTask
                     System.out.println("");
                     System.out.println("Error: " + rootCause.getMessage());
                     System.out.println(
-                        "       Line: " + 
-                            ((SAXParseException)rootCause).getLineNumber() + 
-                        " Column: " + 
+                        "       Line: " +
+                            ((SAXParseException)rootCause).getLineNumber() +
+                        " Column: " +
                             ((SAXParseException)rootCause).getColumnNumber());
                     System.out.println("");
                 }
@@ -397,17 +445,15 @@ public class AnakiaTask extends MatchingTask
             {
                 e.printStackTrace();
             }
-//            log("Failed to process " + inFile, Project.MSG_INFO);
         }
         catch (Throwable e)
         {
-//            log("Failed to process " + inFile, Project.MSG_INFO);
             if (outFile != null)
             {
                 outFile.delete();
             }
             e.printStackTrace();
-        }        
+        }
         finally
         {
             if (writer != null)
@@ -415,15 +461,24 @@ public class AnakiaTask extends MatchingTask
                 try
                 {
                     writer.flush();
+                }
+                catch (IOException e)
+                {
+                    // Do nothing
+                }
+
+                try
+                {
                     writer.close();
                 }
-                catch (Exception e)
+                catch (IOException e)
                 {
+                    // Do nothing
                 }
             }
         }
     }
-    
+
     /**
      * Hacky method to figure out the relative path
      * that we are currently in. This is good for getting
@@ -436,7 +491,7 @@ public class AnakiaTask extends MatchingTask
         StringTokenizer st = new StringTokenizer(file, "/\\");
         // needs to be -1 cause ST returns 1 even if there are no matches. huh?
         int slashCount = st.countTokens() - 1;
-        StringBuffer sb = new StringBuffer();        
+        StringBuffer sb = new StringBuffer();
         for (int i=0;i<slashCount ;i++ )
         {
             sb.append ("../");
@@ -446,12 +501,10 @@ public class AnakiaTask extends MatchingTask
         {
             return StringUtils.chop(sb.toString(), 1);
         }
-        else
-        {
-            return ".";
-        }
+
+        return ".";
     }
-    
+
     /**
      * create directories as needed
      */
@@ -462,9 +515,131 @@ public class AnakiaTask extends MatchingTask
         {
             if (!directory.mkdirs())
             {
-                throw new BuildException("Unable to create directory: " 
+                throw new BuildException("Unable to create directory: "
                                          + directory.getAbsolutePath() );
             }
         }
     }
-}    
+
+
+    /**
+     * Check to see if user context is modified.
+     */
+    private boolean userContextsModifed(long lastModified)
+    {
+        for (Iterator iter = contexts.iterator(); iter.hasNext();)
+        {
+            AnakiaTask.Context ctx = (AnakiaTask.Context) iter.next();
+            if(ctx.getLastModified() > lastModified)
+            {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    /**
+     * Create a new context.
+     * @return A new context.
+     */
+    public Context createContext()
+    {
+        Context context = new Context();
+        contexts.add(context);
+        return context;
+    }
+
+
+    /**
+     * A context implementation that loads all values from an XML file.
+     */
+    public class Context
+    {
+
+        private String name;
+        private Document contextDoc = null;
+        private String file;
+
+        /**
+         * Public constructor.
+         */
+        public Context()
+        {
+        }
+
+        /**
+         * Get the name of the context.
+         * @return The name of the context.
+         */
+        public String getName()
+        {
+            return name;
+        }
+
+        /**
+         * Set the name of the context.
+         * @param name
+         *
+         * @throws IllegalArgumentException if a reserved word is used as a
+         * name, specifically any of "relativePath", "treeWalk", "xpath",
+         * "escape", "date", or "project"
+         */
+        public void setName(String name)
+        {
+            if (name.equals("relativePath") ||
+                    name.equals("treeWalk") ||
+                    name.equals("xpath") ||
+                    name.equals("escape") ||
+                    name.equals("date") ||
+                    name.equals("project"))
+            {
+
+                    throw new IllegalArgumentException("Context name '" + name
+                            + "' is reserved by Anakia");
+            }
+
+            this.name = name;
+        }
+
+        /**
+         * Build the context based on a file path.
+         * @param file
+         */
+        public void setFile(String file)
+        {
+            this.file = file;
+        }
+
+        /**
+         * Retrieve the time the source file was last modified.
+         * @return The time the source file was last modified.
+         */
+        public long getLastModified()
+        {
+            return new File(baseDir, file).lastModified();
+        }
+
+        /**
+         * Retrieve the context document object.
+         * @return The context document object.
+         */
+        public Document getContextDocument()
+        {
+            if (contextDoc == null)
+            {
+                File contextFile = new File(baseDir, file);
+
+                try
+                {
+                    contextDoc = builder.build(contextFile);
+                }
+                catch (Exception e)
+                {
+                    throw new BuildException(e);
+                }
+            }
+            return contextDoc;
+        }
+    }
+
+}
