@@ -1,30 +1,28 @@
 package org.apache.velocity.context;
 
 /*
- * Copyright 2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.    
  */
 
-import java.io.Serializable;
-
-import org.apache.velocity.context.Context;
-import org.apache.velocity.context.InternalContextBase;
-
 /**
- *  This class is the abstract base class for all conventional 
- *  Velocity Context  implementations.  Simply extend this class 
- *  and implement the abstract routines that access your preferred 
+ *  This class is the abstract base class for all conventional
+ *  Velocity Context  implementations.  Simply extend this class
+ *  and implement the abstract routines that access your preferred
  *  storage method.
  *
  *  Takes care of context chaining.
@@ -39,7 +37,7 @@ import org.apache.velocity.context.InternalContextBase;
  *        will be dropped and logged.
  *  </ul>
  *
- *  The default implementation of this for application use is 
+ *  The default implementation of this for application use is
  *  org.apache.velocity.VelocityContext.
  *
  *  All thanks to Fedor for the chaining idea.
@@ -47,33 +45,33 @@ import org.apache.velocity.context.InternalContextBase;
  * @author <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
  * @author <a href="mailto:fedor.karpelevitch@home.com">Fedor Karpelevitch</a>
  * @author <a href="mailto:jvanzyl@apache.org">Jason van Zyl</a>
- * @version $Id: AbstractContext.java,v 1.8.8.1 2004/03/03 23:22:54 geirm Exp $
+ * @version $Id: AbstractContext.java 702218 2008-10-06 18:15:18Z nbubna $
  */
 
-public abstract class AbstractContext extends InternalContextBase 
-    implements Context, Serializable
+public abstract class AbstractContext extends InternalContextBase
+    implements Context
 {
     /**
      *  the chained Context if any
      */
     private   Context  innerContext = null;
-  
-    /** 
-     *  Implement to return a value from the context storage. 
+
+    /**
+     *  Implement to return a value from the context storage.
      *  <br><br>
      *  The implementation of this method is required for proper
      *  operation of a Context implementation in general
      *  Velocity use.
-     *  
+     *
      *  @param key key whose associated value is to be returned
      *  @return object stored in the context
      */
     public abstract Object internalGet( String key );
 
-    /** 
+    /**
      *  Implement to put a value into the context storage.
      *  <br><br>
-     *  The implementation of this method is required for 
+     *  The implementation of this method is required for
      *  proper operation of a Context implementation in
      *  general Velocity use.
      *
@@ -83,33 +81,33 @@ public abstract class AbstractContext extends InternalContextBase
      */
     public abstract Object internalPut( String key, Object value );
 
-    /** 
+    /**
      *  Implement to determine if a key is in the storage.
      *  <br><br>
-     *  Currently, this method is not used internally by 
-     *  the Velocity core. 
+     *  Currently, this method is not used internally by
+     *  the Velocity engine.
      *
-     *   @param key key to test for existance 
+     *   @param key key to test for existance
      *   @return true if found, false if not
      */
     public abstract boolean internalContainsKey(Object key);
 
-    /** 
-     *  Implement to return an object array of key 
+    /**
+     *  Implement to return an object array of key
      *  strings from your storage.
      *  <br><br>
      *  Currently, this method is not used internally by
-     *  the Velocity core.
+     *  the Velocity engine.
      *
      *  @return array of keys
      */
     public abstract Object[] internalGetKeys();
 
-    /** 
+    /**
      *  I mplement to remove an item from your storage.
      *  <br><br>
      *  Currently, this method is not used internally by
-     *  the Velocity core.
+     *  the Velocity engine.
      *
      *  @param key key to remove
      *  @return object removed if exists, else null
@@ -121,13 +119,13 @@ public abstract class AbstractContext extends InternalContextBase
      */
     public AbstractContext()
     {
-    }        
+    }
 
     /**
      *  Chaining constructor accepts a Context argument.
      *  It will relay get() operations into this Context
      *  in the even the 'local' get() returns null.
-     *  
+     *
      *  @param inner context to be chained
      */
     public AbstractContext( Context inner )
@@ -146,8 +144,8 @@ public abstract class AbstractContext extends InternalContextBase
     }
 
     /**
-     * Adds a name/value pair to the context. 
-     * 
+     * Adds a name/value pair to the context.
+     *
      * @param key   The name to key the provided value with.
      * @param value The corresponding value.
      * @return Object that was replaced in the the Context if
@@ -156,17 +154,12 @@ public abstract class AbstractContext extends InternalContextBase
     public Object put(String key, Object value)
     {
         /*
-         * don't even continue if key or value is null
+         * don't even continue if key is null
          */
-
         if (key == null)
         {
             return null;
         }
-        else if (value == null)
-        {
-            return null;
-        }
         
         return internalPut(key, value);
     }
@@ -203,12 +196,12 @@ public abstract class AbstractContext extends InternalContextBase
         {
             o = innerContext.get( key );
         }
-            
+
         return o;
-    }        
+    }
 
     /**
-     *  Indicates whether the specified key is in the context.  Provided for 
+     *  Indicates whether the specified key is in the context.  Provided for
      *  debugging purposes.
      *
      * @param key The key to look for.
@@ -221,8 +214,14 @@ public abstract class AbstractContext extends InternalContextBase
             return false;
         }
 
-        return internalContainsKey(key);
-    }        
+        boolean exists = internalContainsKey(key);
+        if (!exists && innerContext != null)
+        {
+            exists = innerContext.containsKey(key);
+        }
+        
+        return exists;
+    }
 
     /**
      *  Get all the keys for the values in the context
@@ -238,7 +237,7 @@ public abstract class AbstractContext extends InternalContextBase
      * Removes the value associated with the specified key from the context.
      *
      * @param key The name of the value to remove.
-     * @return    The value that the key was mapped to, or <code>null</code> 
+     * @return    The value that the key was mapped to, or <code>null</code>
      *            if unmapped.
      */
     public Object remove(Object key)
@@ -249,7 +248,7 @@ public abstract class AbstractContext extends InternalContextBase
         }
 
         return internalRemove(key);
-    }   
+    }
 
     /**
      *  returns innerContext if one is chained
