@@ -1,122 +1,65 @@
 package org.apache.velocity.runtime;
 
 /*
- * The Apache Software License, Version 1.1
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Velocity", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.    
  */
 
-import java.io.InputStream;
-import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.Reader;
-
-import java.util.List;
-import java.util.ArrayList;
-import java.util.Map;
-import java.util.Hashtable;
+import java.io.Writer;
 import java.util.Properties;
-import java.util.Stack;
-import java.util.Enumeration;
-import java.util.TreeMap;
-import java.util.Vector;
-
-import org.apache.log.Logger;
-
-import org.apache.velocity.Template;
-
-import org.apache.velocity.runtime.log.LogManager;
-import org.apache.velocity.runtime.log.LogSystem;
-
-import org.apache.velocity.runtime.parser.Parser;
-import org.apache.velocity.runtime.parser.ParseException;
-import org.apache.velocity.runtime.parser.node.SimpleNode;
-
-import org.apache.velocity.runtime.directive.Directive;
-import org.apache.velocity.runtime.VelocimacroFactory;
-
-import org.apache.velocity.runtime.resource.Resource;
-import org.apache.velocity.runtime.resource.ContentResource;
-import org.apache.velocity.runtime.resource.ResourceManager;
-
-import org.apache.velocity.util.SimplePool;
-import org.apache.velocity.util.StringUtils;
-
-import org.apache.velocity.util.introspection.Introspector;
-
-import org.apache.velocity.exception.ResourceNotFoundException;
-import org.apache.velocity.exception.ParseErrorException;
 
 import org.apache.commons.collections.ExtendedProperties;
+import org.apache.velocity.Template;
+import org.apache.velocity.app.event.EventCartridge;
+import org.apache.velocity.context.Context;
+import org.apache.velocity.exception.MethodInvocationException;
+import org.apache.velocity.exception.ParseErrorException;
+import org.apache.velocity.exception.ResourceNotFoundException;
+import org.apache.velocity.runtime.directive.Directive;
+import org.apache.velocity.runtime.log.Log;
+import org.apache.velocity.runtime.parser.ParseException;
+import org.apache.velocity.runtime.parser.Parser;
+import org.apache.velocity.runtime.parser.node.Node;
+import org.apache.velocity.runtime.parser.node.SimpleNode;
+import org.apache.velocity.runtime.resource.ContentResource;
+import org.apache.velocity.util.introspection.Introspector;
+import org.apache.velocity.util.introspection.Uberspect;
 
 
 /**
- * Interface for internal runtime services that are needed by the 
+ * Interface for internal runtime services that are needed by the
  * various components w/in Velocity.  This was taken from the old
  * Runtime singleton, and anything not necessary was removed.
  *
  *  Currently implemented by RuntimeInstance.
- * 
+ *
  * @author <a href="mailto:geirm@optonline.net">Geir Magusson Jr.</a>
- * @version $Id: RuntimeServices.java,v 1.4 2002/01/03 12:40:23 geirm Exp $
+ * @version $Id: RuntimeServices.java 685724 2008-08-13 23:12:12Z nbubna $
  */
-public interface RuntimeServices
+public interface RuntimeServices extends RuntimeLogger
 {
 
-   /*
+   /**
      * This is the primary initialization method in the Velocity
      * Runtime. The systems that are setup/initialized here are
      * as follows:
-     * 
+     *
      * <ul>
      *   <li>Logging System</li>
      *   <li>ResourceManager</li>
@@ -125,6 +68,7 @@ public interface RuntimeServices
      *   <li>Static Content Include System</li>
      *   <li>Velocimacro System</li>
      * </ul>
+     * @throws Exception
      */
     public void init() throws Exception;
 
@@ -132,8 +76,8 @@ public interface RuntimeServices
      * Allows an external system to set a property in
      * the Velocity Runtime.
      *
-     * @param String property key
-     * @param String property value
+     * @param key property key
+     * @param value property value
      */
     public  void setProperty(String key, Object value);
 
@@ -145,7 +89,7 @@ public interface RuntimeServices
      * parent application's configuration. This is
      * the case with Turbine.
      *
-     * @param ExtendedProperties configuration
+     * @param configuration
      */
     public void setConfiguration( ExtendedProperties configuration);
 
@@ -165,25 +109,26 @@ public interface RuntimeServices
      *
      * ["file", "classpath"]
      *
-     * @param String key
-     * @param String value
+     * @param key
+     * @param value
      */
     public void addProperty(String key, Object value);
-    
+
     /**
      * Clear the values pertaining to a particular
      * property.
      *
-     * @param String key of property to clear
+     * @param key of property to clear
      */
     public void clearProperty(String key);
-    
+
     /**
      *  Allows an external caller to get a property.  The calling
      *  routine is required to know the type, as this routine
      *  will return an Object, as that is what properties can be.
      *
      *  @param key property to return
+     * @return The value.
      */
     public  Object getProperty( String key );
 
@@ -191,32 +136,44 @@ public interface RuntimeServices
      * Initialize the Velocity Runtime with a Properties
      * object.
      *
-     * @param Properties
+     * @param p
+     * @throws Exception
      */
     public void init(Properties p) throws Exception;
-    
+
     /**
      * Initialize the Velocity Runtime with the name of
      * ExtendedProperties object.
      *
-     * @param Properties
+     * @param configurationFile
+     * @throws Exception
      */
     public void init(String configurationFile) throws Exception;
 
+    /**
+     * Wraps the String in a StringReader and passes it off to
+     * {@link #parse(Reader,String)}.
+     * @since 1.6
+     */
+    public SimpleNode parse(String string, String templateName)
+        throws ParseException;
+
     /**
      * Parse the input and return the root of
      * AST node structure.
      * <br><br>
      *  In the event that it runs out of parsers in the
-     *  pool, it will create and let them be GC'd 
+     *  pool, it will create and let them be GC'd
      *  dynamically, logging that it has to do that.  This
      *  is considered an exceptional condition.  It is
-     *  expected that the user will set the 
+     *  expected that the user will set the
      *  PARSER_POOL_SIZE property appropriately for their
      *  application.  We will revisit this.
      *
-     * @param InputStream inputstream retrieved by a resource loader
-     * @param String name of the template being parsed
+     * @param reader inputstream retrieved by a resource loader
+     * @param templateName name of the template being parsed
+     * @return The AST representing the template.
+     * @throws ParseException
      */
     public  SimpleNode parse( Reader reader, String templateName )
         throws ParseException;
@@ -224,16 +181,84 @@ public interface RuntimeServices
     /**
      *  Parse the input and return the root of the AST node structure.
      *
-     * @param InputStream inputstream retrieved by a resource loader
-     * @param String name of the template being parsed
+     * @param reader inputstream retrieved by a resource loader
+     * @param templateName name of the template being parsed
      * @param dumpNamespace flag to dump the Velocimacro namespace for this template
+     * @return The AST representing the template.
+     * @throws ParseException
      */
     public SimpleNode parse( Reader reader, String templateName, boolean dumpNamespace )
         throws ParseException;
-    
+
+    /**
+     * Renders the input string using the context into the output writer.
+     * To be used when a template is dynamically constructed, or want to use
+     * Velocity as a token replacer.
+     *
+     * @param context context to use in rendering input string
+     * @param out  Writer in which to render the output
+     * @param logTag  string to be used as the template name for log
+     *                messages in case of error
+     * @param instring input string containing the VTL to be rendered
+     *
+     * @return true if successful, false otherwise.  If false, see
+     *              Velocity runtime log
+     * @throws ParseErrorException The template could not be parsed.
+     * @throws MethodInvocationException A method on a context object could not be invoked.
+     * @throws ResourceNotFoundException A referenced resource could not be loaded.
+     * @throws IOException While rendering to the writer, an I/O problem occured.
+     * @since Velocity 1.6
+     */
+    public boolean evaluate(Context context, Writer out,
+                            String logTag, String instring) throws IOException;
+
+    /**
+     * Renders the input reader using the context into the output writer.
+     * To be used when a template is dynamically constructed, or want to
+     * use Velocity as a token replacer.
+     *
+     * @param context context to use in rendering input string
+     * @param writer  Writer in which to render the output
+     * @param logTag  string to be used as the template name for log messages
+     *                in case of error
+     * @param reader Reader containing the VTL to be rendered
+     *
+     * @return true if successful, false otherwise.  If false, see
+     *              Velocity runtime log
+     * @throws ParseErrorException The template could not be parsed.
+     * @throws MethodInvocationException A method on a context object could not be invoked.
+     * @throws ResourceNotFoundException A referenced resource could not be loaded.
+     * @throws IOException While reading from the reader or rendering to the writer,
+     *                     an I/O problem occured.
+     * @since Velocity 1.6
+     */
+    public boolean evaluate(Context context, Writer writer,
+                            String logTag, Reader reader) throws IOException;
+
+    /**
+     * Invokes a currently registered Velocimacro with the params provided
+     * and places the rendered stream into the writer.
+     * <br>
+     * Note : currently only accepts args to the VM if they are in the context.
+     *
+     * @param vmName name of Velocimacro to call
+     * @param logTag string to be used for template name in case of error. if null,
+     *               the vmName will be used
+     * @param params keys for args used to invoke Velocimacro, in java format
+     *               rather than VTL (eg  "foo" or "bar" rather than "$foo" or "$bar")
+     * @param context Context object containing data/objects used for rendering.
+     * @param writer  Writer for output stream
+     * @return true if Velocimacro exists and successfully invoked, false otherwise.
+     * @throws IOException While rendering to the writer, an I/O problem occured.
+     * @since 1.6
+     */
+    public boolean invokeVelocimacro(final String vmName, String logTag,
+                                     String[] params, final Context context,
+                                     final Writer writer) throws IOException;
+
     /**
      * Returns a <code>Template</code> from the resource manager.
-     * This method assumes that the character encoding of the 
+     * This method assumes that the character encoding of the
      * template is set by the <code>input.encoding</code>
      * property.  The default is "ISO-8859-1"
      *
@@ -272,6 +297,8 @@ public interface RuntimeServices
      * @return parsed ContentResource object ready for use
      * @throws ResourceNotFoundException if template not found
      *          from any available source.
+     * @throws ParseErrorException
+     * @throws Exception
      */
     public ContentResource getContent(String name)
         throws ResourceNotFoundException, ParseErrorException, Exception;
@@ -285,12 +312,14 @@ public interface RuntimeServices
      * @return parsed ContentResource object ready for use
      * @throws ResourceNotFoundException if template not found
      *          from any available source.
+     * @throws ParseErrorException
+     * @throws Exception
      */
     public ContentResource getContent( String name, String encoding )
         throws ResourceNotFoundException, ParseErrorException, Exception;
 
     /**
-     *  Determines is a template exists, and returns name of the loader that 
+     *  Determines is a template exists, and returns name of the loader that
      *  provides it.  This is a slightly less hokey way to support
      *  the Velocity.templateExists() utility method, which was broken
      *  when per-template encoding was introduced.  We can revisit this.
@@ -299,43 +328,15 @@ public interface RuntimeServices
      *  @return class name of loader than can provide it
      */
     public String getLoaderNameForResource( String resourceName );
-    
-    /**
-     * Log a warning message.
-     *
-     * @param Object message to log
-     */
-    public void warn(Object message);
-    
-    /** 
-     * Log an info message.
-     *
-     * @param Object message to log
-     */
-    public  void info(Object message);
-    
-    /**
-     * Log an error message.
-     *
-     * @param Object message to log
-     */
-    public void error(Object message);
-    
-    /**
-     * Log a debug message.
-     *
-     * @param Object message to log
-     */
-    public void debug(Object message);
 
     /**
      * String property accessor method with default to hide the
      * configuration implementation.
-     * 
-     * @param String key property key
-     * @param String defaultValue  default value to return if key not 
+     *
+     * @param key property key
+     * @param defaultValue  default value to return if key not
      *               found in resource manager.
-     * @return String  value of key or default 
+     * @return String  value of key or default
      */
     public String getString( String key, String defaultValue);
 
@@ -343,30 +344,72 @@ public interface RuntimeServices
      * Returns the appropriate VelocimacroProxy object if strVMname
      * is a valid current Velocimacro.
      *
-     * @param String vmName  Name of velocimacro requested
-     * @return String VelocimacroProxy 
+     * @param vmName  Name of velocimacro requested
+     * @param templateName Name of the namespace.
+     * @return VelocimacroProxy
      */
     public Directive getVelocimacro( String vmName, String templateName  );
+    
+    /**
+     * Returns the appropriate VelocimacroProxy object if strVMname
+     * is a valid current Velocimacro.
+     *
+     * @param vmName  Name of velocimacro requested
+     * @param templateName Name of the namespace.
+     * @param renderingTemplate Name of the template we are currently rendering. This
+     *    information is needed when VM_PERM_ALLOW_INLINE_REPLACE_GLOBAL setting is true
+     *    and template contains a macro with the same name as the global macro library.
+     * 
+     * @since Velocity 1.6
+     * 
+     * @return VelocimacroProxy
+     */
+    public Directive getVelocimacro( String vmName, String templateName, String renderingTemplate  );
 
    /**
      * Adds a new Velocimacro. Usually called by Macro only while parsing.
      *
-     * @param String name  Name of velocimacro 
-     * @param String macro  String form of macro body
-     * @param String argArray  Array of strings, containing the 
+     * @param name  Name of velocimacro
+     * @param macro  String form of macro body
+     * @param argArray  Array of strings, containing the
      *                         #macro() arguments.  the 0th is the name.
-     * @return boolean  True if added, false if rejected for some 
-     *                  reason (either parameters or permission settings) 
+     * @param sourceTemplate
+     * 
+     * @deprecated Use addVelocimacro(String, Node, String[], String) instead
+     *                   
+     * @return boolean  True if added, false if rejected for some
+     *                  reason (either parameters or permission settings)
      */
-    public boolean addVelocimacro( String name, 
-                                          String macro, 
-                                          String argArray[], 
+    public boolean addVelocimacro( String name,
+                                          String macro,
+                                          String argArray[],
                                           String sourceTemplate );
- 
+
+    /**
+     * Adds a new Velocimacro. Usually called by Macro only while parsing.
+     *
+     * @param name  Name of velocimacro
+     * @param macro  root AST node of the parsed macro
+     * @param argArray  Array of strings, containing the
+     *                         #macro() arguments.  the 0th is the name.
+     * @param sourceTemplate
+     * 
+     * @since Velocity 1.6
+     *                   
+     * @return boolean  True if added, false if rejected for some
+     *                  reason (either parameters or permission settings)
+     */
+    public boolean addVelocimacro( String name,
+                                          Node macro,
+                                          String argArray[],
+                                          String sourceTemplate );
+                                          
+                                          
     /**
      *  Checks to see if a VM exists
      *
-     * @param name  Name of velocimacro
+     * @param vmName  Name of velocimacro
+     * @param templateName
      * @return boolean  True if VM by that name exists, false if not
      */
     public boolean isVelocimacro( String vmName, String templateName );
@@ -374,6 +417,8 @@ public interface RuntimeServices
     /**
      *  tells the vmFactory to dump the specified namespace.  This is to support
      *  clearing the VM list when in inline-VM-local-scope mode
+     * @param namespace
+     * @return True if the Namespace was dumped.
      */
     public boolean dumpVMNamespace( String namespace );
 
@@ -387,7 +432,7 @@ public interface RuntimeServices
     /**
      * Int property accessor method to hide the configuration implementation.
      *
-     * @param String key property key
+     * @param key property key
      * @return int value
      */
     public int getInt( String key );
@@ -396,16 +441,16 @@ public interface RuntimeServices
      * Int property accessor method to hide the configuration implementation.
      *
      * @param key  property key
-     * @param int default value
+     * @param defaultValue default value
      * @return int  value
      */
     public int getInt( String key, int defaultValue );
 
     /**
      * Boolean property accessor method to hide the configuration implementation.
-     * 
-     * @param String key  property key
-     * @param boolean default default value if property not found
+     *
+     * @param key  property key
+     * @param def default default value if property not found
      * @return boolean  value of key or default value
      */
     public boolean getBoolean( String key, boolean def );
@@ -418,14 +463,69 @@ public interface RuntimeServices
      */
     public ExtendedProperties getConfiguration();
 
-    /*
-     *  Return this instance's Introspector
-     */
-    public Introspector getIntrospector();
-    
-    /*
-     *  Return the specified applcation attribute
+    /**
+     * Return the specified application attribute.
+     *
+     * @param key The name of the attribute to retrieve.
+     * @return The value of the attribute.
      */
     public Object getApplicationAttribute( Object key );
-    
+
+    /**
+     * Set the specified application attribute.
+     *
+     * @param key The name of the attribute to set.
+     * @param value The attribute value to set.
+     * @return the displaced attribute value
+     */
+    public Object setApplicationAttribute( Object key, Object value );
+
+    /**
+     * Returns the configured class introspection/reflection
+     * implementation.
+     * @return The current Uberspect object.
+     */
+    public Uberspect getUberspect();
+
+    /**
+     * Returns a convenient Log instance that wraps the current LogChute.
+     * @return A log object.
+     */
+    public Log getLog();
+
+    /**
+     * Returns the event handlers for the application.
+     * @return The event handlers for the application.
+     */
+    public EventCartridge getApplicationEventCartridge();
+
+
+    /**
+     * Returns the configured method introspection/reflection
+     * implementation.
+     * @return The configured method introspection/reflection
+     * implementation.
+     */
+    public Introspector getIntrospector();
+
+    /**
+     * Returns true if the RuntimeInstance has been successfully initialized.
+     * @return True if the RuntimeInstance has been successfully initialized.
+     */
+    public boolean isInitialized();
+
+    /**
+     * Create a new parser instance.
+     * @return A new parser instance.
+     */
+    public Parser createNewParser();
+
+    /**
+     * Retrieve a previously instantiated directive.
+     * @param name name of the directive
+     * @return the directive with that name, if any
+     * @since 1.6
+     */
+    public Directive getDirective(String name);
+
 }
