@@ -1,36 +1,39 @@
 package org.apache.velocity.runtime.parser.node;
 
 /*
- * Copyright 2000-2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.    
  */
 
-import org.apache.velocity.runtime.parser.Parser;
-import org.apache.velocity.runtime.parser.Token;
-import org.apache.velocity.context.InternalContextAdapter;
-import org.apache.velocity.exception.ResourceNotFoundException;
-import org.apache.velocity.exception.ParseErrorException;
-import org.apache.velocity.exception.MethodInvocationException;
-
 import java.io.IOException;
 import java.io.Writer;
 
+import org.apache.velocity.context.InternalContextAdapter;
+import org.apache.velocity.exception.MethodInvocationException;
+import org.apache.velocity.exception.ParseErrorException;
+import org.apache.velocity.exception.ResourceNotFoundException;
+import org.apache.velocity.runtime.parser.Parser;
+import org.apache.velocity.runtime.parser.Token;
+
 /**
  *  Represents all comments...
  *
  *  @author <a href="mailto:geirm@apache.org">Geir Magnusson Jr.</a>
- *  @version $Id: ASTComment.java,v 1.5.4.1 2004/03/03 23:22:58 geirm Exp $
+ *  @version $Id: ASTComment.java 517553 2007-03-13 06:09:58Z wglass $
  */
 public class ASTComment extends SimpleNode
 {
@@ -38,27 +41,38 @@ public class ASTComment extends SimpleNode
 
     private char[] carr;
 
+    /**
+     * @param id
+     */
     public ASTComment(int id)
     {
         super(id);
     }
 
-    public ASTComment(Parser p, int id) 
+    /**
+     * @param p
+     * @param id
+     */
+    public ASTComment(Parser p, int id)
     {
         super(p, id);
     }
 
-    /** Accept the visitor. **/
-    public Object jjtAccept(ParserVisitor visitor, Object data) 
+    /**
+     * @see org.apache.velocity.runtime.parser.node.SimpleNode#jjtAccept(org.apache.velocity.runtime.parser.node.ParserVisitor, java.lang.Object)
+     */
+    public Object jjtAccept(ParserVisitor visitor, Object data)
     {
         return visitor.visit(this, data);
     }
 
     /**
      *  We need to make sure we catch any of the dreaded MORE tokens.
+     * @param context
+     * @param data
+     * @return The data object.
      */
     public Object init(InternalContextAdapter context, Object data)
-            throws Exception
     {
         Token t = getFirstToken();
 
@@ -77,10 +91,17 @@ public class ASTComment extends SimpleNode
         return data;
     }
 
+    /**
+     * @see org.apache.velocity.runtime.parser.node.SimpleNode#render(org.apache.velocity.context.InternalContextAdapter, java.io.Writer)
+     */
     public boolean render( InternalContextAdapter context, Writer writer)
         throws IOException, MethodInvocationException, ParseErrorException, ResourceNotFoundException
     {
-        writer.write(carr);
+
+        if (context.getAllowRendering())
+        {
+            writer.write(carr);
+        }
 
         return true;
     }
