@@ -1,32 +1,37 @@
 package org.apache.velocity.runtime.parser.node;
 
 /*
- * Copyright 2000-2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.    
  */
 
-import org.apache.velocity.context.InternalContextAdapter;
-import org.apache.velocity.runtime.parser.Parser;
-import org.apache.velocity.util.introspection.IntrospectionCacheData;
-import org.apache.velocity.util.introspection.Info;
-import org.apache.velocity.util.introspection.VelPropertyGet;
-
-import org.apache.velocity.exception.MethodInvocationException;
-import org.apache.velocity.app.event.EventCartridge;
-
 import java.lang.reflect.InvocationTargetException;
 
+import org.apache.velocity.app.event.EventHandlerUtil;
+import org.apache.velocity.context.InternalContextAdapter;
+import org.apache.velocity.exception.MethodInvocationException;
+import org.apache.velocity.exception.TemplateInitException;
+import org.apache.velocity.exception.VelocityException;
+import org.apache.velocity.runtime.RuntimeConstants;
+import org.apache.velocity.runtime.parser.Parser;
+import org.apache.velocity.util.introspection.Info;
+import org.apache.velocity.util.introspection.IntrospectionCacheData;
+import org.apache.velocity.util.introspection.VelPropertyGet;
+
 /**
  *  ASTIdentifier.java
  *
@@ -41,7 +46,7 @@ import java.lang.reflect.InvocationTargetException;
  *
  * @author <a href="mailto:jvanzyl@apache.org">Jason van Zyl</a>
  * @author <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
- * @version $Id: ASTIdentifier.java,v 1.19.4.1 2004/03/03 23:22:58 geirm Exp $
+ * @version $Id: ASTIdentifier.java 720228 2008-11-24 16:58:33Z nbubna $
  */
 public class ASTIdentifier extends SimpleNode
 {
@@ -51,18 +56,33 @@ public class ASTIdentifier extends SimpleNode
      *  This is really immutable after the init, so keep one for this node
      */
     protected Info uberInfo;
+    
+    /**
+     * Indicates if we are running in strict reference mode.
+     */
+    protected boolean strictRef = false;
 
+    /**
+     * @param id
+     */
     public ASTIdentifier(int id)
     {
         super(id);
     }
 
+    /**
+     * @param p
+     * @param id
+     */
     public ASTIdentifier(Parser p, int id)
     {
         super(p, id);
     }
 
-    /** Accept the visitor. **/
+
+    /**
+     * @see org.apache.velocity.runtime.parser.node.SimpleNode#jjtAccept(org.apache.velocity.runtime.parser.node.ParserVisitor, java.lang.Object)
+     */
     public Object jjtAccept(ParserVisitor visitor, Object data)
     {
         return visitor.visit(this, data);
@@ -71,22 +91,27 @@ public class ASTIdentifier extends SimpleNode
     /**
      *  simple init - don't do anything that is context specific.
      *  just get what we need from the AST, which is static.
+     * @param context
+     * @param data
+     * @return The data object.
+     * @throws TemplateInitException
      */
     public  Object init(InternalContextAdapter context, Object data)
-        throws Exception
+        throws TemplateInitException
     {
         super.init(context, data);
 
         identifier = getFirstToken().image;
 
-        uberInfo = new Info(context.getCurrentTemplateName(),
-                getLine(), getColumn());
+        uberInfo = new Info(getTemplateName(), getLine(), getColumn());
 
+        strictRef = rsvc.getBoolean(RuntimeConstants.RUNTIME_REFERENCES_STRICT, false);
+        
         return data;
     }
 
     /**
-     *  invokes the method on the object passed in
+     * @see org.apache.velocity.runtime.parser.node.SimpleNode#execute(java.lang.Object, org.apache.velocity.context.InternalContextAdapter)
      */
     public Object execute(Object o, InternalContextAdapter context)
         throws MethodInvocationException
@@ -96,8 +121,6 @@ public class ASTIdentifier extends SimpleNode
 
         try
         {
-            Class c = o.getClass();
-
             /*
              *  first, see if we have this information cached.
              */
@@ -111,7 +134,7 @@ public class ASTIdentifier extends SimpleNode
              * that is fixed in the template :)
              */
 
-            if (icd != null && icd.contextData == c)
+            if ( icd != null && (o != null) && (icd.contextData == o.getClass()) )
             {
                 vg = (VelPropertyGet) icd.thingy;
             }
@@ -124,19 +147,28 @@ public class ASTIdentifier extends SimpleNode
 
                 vg = rsvc.getUberspect().getPropertyGet(o,identifier, uberInfo);
 
-                if (vg != null && vg.isCacheable())
+                if (vg != null && vg.isCacheable() && (o != null))
                 {
                     icd = new IntrospectionCacheData();
-                    icd.contextData = c;
+                    icd.contextData = o.getClass();
                     icd.thingy = vg;
                     context.icachePut(this,icd);
                 }
             }
         }
+
+        /**
+         * pass through application level runtime exceptions
+         */
+        catch( RuntimeException e )
+        {
+            throw e;
+        }
         catch(Exception e)
         {
-            rsvc.error("ASTIdentifier.execute() : identifier = "
-                               + identifier + " : " + e);
+            String msg = "ASTIdentifier.execute() : identifier = "+identifier;
+            log.error(msg, e);
+            throw new VelocityException(msg, e);
         }
 
         /*
@@ -145,7 +177,16 @@ public class ASTIdentifier extends SimpleNode
 
         if (vg == null)
         {
-            return null;
+            if (strictRef)
+            {
+                throw new MethodInvocationException("Object '" + o.getClass().getName() +              
+                    "' does not contain property '" + identifier + "'", null, identifier,
+                    uberInfo.getTemplateName(), uberInfo.getLine(), uberInfo.getColumn());
+            }
+            else
+            {
+                return null;
+            }
         }
 
         /*
@@ -158,30 +199,33 @@ public class ASTIdentifier extends SimpleNode
         }
         catch(InvocationTargetException ite)
         {
-            EventCartridge ec = context.getEventCartridge();
-
             /*
              *  if we have an event cartridge, see if it wants to veto
              *  also, let non-Exception Throwables go...
              */
 
-            if (ec != null
-                    && ite.getTargetException() instanceof java.lang.Exception)
+            Throwable t = ite.getTargetException();
+            if (t instanceof Exception)
             {
                 try
                 {
-                    return ec.methodException(o.getClass(), vg.getMethodName(),
-                            (Exception)ite.getTargetException());
+                    return EventHandlerUtil.methodException(rsvc, context, o.getClass(), vg.getMethodName(),
+                            (Exception) t);
                 }
-                catch(Exception e)
+
+                /**
+                 * If the event handler throws an exception, then wrap it
+                 * in a MethodInvocationException.  Don't pass through RuntimeExceptions like other
+                 * similar catchall code blocks.
+                 */
+                catch( Exception e )
                 {
                     throw new MethodInvocationException(
                       "Invocation of method '" + vg.getMethodName() + "'"
                       + " in  " + o.getClass()
                       + " threw exception "
-                      + ite.getTargetException().getClass() + " : "
-                      + ite.getTargetException().getMessage(),
-                      ite.getTargetException(), vg.getMethodName());
+                      + ite.getTargetException().toString(),
+                      ite.getTargetException(), vg.getMethodName(), getTemplateName(), this.getLine(), this.getColumn());
                 }
             }
             else
@@ -194,9 +238,8 @@ public class ASTIdentifier extends SimpleNode
                 "Invocation of method '" + vg.getMethodName() + "'"
                 + " in  " + o.getClass()
                 + " threw exception "
-                + ite.getTargetException().getClass() + " : "
-                + ite.getTargetException().getMessage(),
-                ite.getTargetException(), vg.getMethodName());
+                + ite.getTargetException().toString(),
+                ite.getTargetException(), vg.getMethodName(), getTemplateName(), this.getLine(), this.getColumn());
 
 
             }
@@ -205,13 +248,20 @@ public class ASTIdentifier extends SimpleNode
         {
             return null;
         }
+        /**
+         * pass through application level runtime exceptions
+         */
+        catch( RuntimeException e )
+        {
+            throw e;
+        }
         catch(Exception e)
         {
-            rsvc.error("ASTIdentifier() : exception invoking method "
+            String msg = "ASTIdentifier() : exception invoking method "
                         + "for identifier '" + identifier + "' in "
-                        + o.getClass() + " : " + e);
+                        + o.getClass();
+            log.error(msg, e);
+            throw new VelocityException(msg, e);
         }
-
-        return null;
     }
 }
