@@ -1,37 +1,41 @@
 package org.apache.velocity.runtime.parser.node;
 
 /*
- * Copyright 2000-2002,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.    
  */
 
-import java.io.Writer;
 import java.io.IOException;
+import java.io.Writer;
 import java.lang.reflect.InvocationTargetException;
 
+import org.apache.velocity.app.event.EventHandlerUtil;
 import org.apache.velocity.context.Context;
 import org.apache.velocity.context.InternalContextAdapter;
-import org.apache.velocity.runtime.RuntimeConstants;
-import org.apache.velocity.runtime.exception.ReferenceException;
-import org.apache.velocity.runtime.parser.*;
-
-import org.apache.velocity.util.introspection.VelPropertySet;
-import org.apache.velocity.util.introspection.Info;
-
 import org.apache.velocity.exception.MethodInvocationException;
-
-import org.apache.velocity.app.event.EventCartridge;
+import org.apache.velocity.exception.TemplateInitException;
+import org.apache.velocity.exception.VelocityException;
+import org.apache.velocity.runtime.RuntimeConstants;
+import org.apache.velocity.runtime.Renderable;
+import org.apache.velocity.runtime.log.Log;
+import org.apache.velocity.runtime.parser.Parser;
+import org.apache.velocity.runtime.parser.Token;
+import org.apache.velocity.util.introspection.Info;
+import org.apache.velocity.util.introspection.VelPropertySet;
 
 /**
  * This class is responsible for handling the references in
@@ -44,7 +48,7 @@ import org.apache.velocity.app.event.EventCartridge;
  * @author <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
  * @author <a href="mailto:Christoph.Reck@dlr.de">Christoph Reck</a>
  * @author <a href="mailto:kjohnson@transparent.com>Kent Johnson</a>
- * @version $Id: ASTReference.java,v 1.49.4.1 2004/03/03 23:22:59 geirm Exp $
+ * @version $Id: ASTReference.java 730988 2009-01-03 14:15:56Z byron $
 */
 public class ASTReference extends SimpleNode
 {
@@ -59,34 +63,52 @@ public class ASTReference extends SimpleNode
     private String rootString;
     private boolean escaped = false;
     private boolean computableReference = true;
+    private boolean logOnNull = true;
     private String escPrefix = "";
     private String morePrefix = "";
     private String identifier = "";
 
     private String literal = null;
 
+    /**
+     * Indicates if we are running in strict reference mode.
+     */
+    public boolean strictRef = false;
+    
     private int numChildren = 0;
 
     protected Info uberInfo;
 
+    /**
+     * @param id
+     */
     public ASTReference(int id)
     {
         super(id);
     }
 
+    /**
+     * @param p
+     * @param id
+     */
     public ASTReference(Parser p, int id)
     {
         super(p, id);
     }
 
-    /** Accept the visitor. **/
+    /**
+     * @see org.apache.velocity.runtime.parser.node.SimpleNode#jjtAccept(org.apache.velocity.runtime.parser.node.ParserVisitor, java.lang.Object)
+     */
     public Object jjtAccept(ParserVisitor visitor, Object data)
     {
         return visitor.visit(this, data);
     }
 
+    /**
+     * @see org.apache.velocity.runtime.parser.node.SimpleNode#init(org.apache.velocity.context.InternalContextAdapter, java.lang.Object)
+     */
     public Object init(InternalContextAdapter context, Object data)
-        throws Exception
+    throws TemplateInitException
     {
         /*
          *  init our children
@@ -117,14 +139,51 @@ public class ASTReference extends SimpleNode
          * make an uberinfo - saves new's later on
          */
 
-        uberInfo = new Info(context.getCurrentTemplateName(),
-                getLine(),getColumn());
+        uberInfo = new Info(getTemplateName(), getLine(),getColumn());
 
+        /*
+         * track whether we log invalid references
+         */
+        logOnNull =
+            rsvc.getBoolean(RuntimeConstants.RUNTIME_LOG_REFERENCE_LOG_INVALID, true);
+
+        strictRef = rsvc.getBoolean(RuntimeConstants.RUNTIME_REFERENCES_STRICT, false);
+ 
+        /**
+         * In the case we are referencing a variable with #if($foo) or
+         * #if( ! $foo) then we allow variables to be undefined and we 
+         * set strictRef to false so that if the variable is undefined
+         * an exception is not thrown. 
+         */
+        if (strictRef && numChildren == 0)
+        {
+            logOnNull = false; // Strict mode allows nulls
+            
+            Node node = this.jjtGetParent();
+            if (node instanceof ASTNotNode     // #if( ! $foo)
+             || node instanceof ASTExpression  // #if( $foo )
+             || node instanceof ASTOrNode      // #if( $foo || ...
+             || node instanceof ASTAndNode)    // #if( $foo && ...
+            {
+                // Now scan up tree to see if we are in an If statement
+                while (node != null)
+                {
+                    if (node instanceof ASTIfStatement)
+                    {
+                       strictRef = false;
+                       break;
+                    }
+                    node = node.jjtGetParent();
+                }
+            }
+        }
+                
         return data;
     }
 
     /**
      *  Returns the 'root string', the reference key
+     * @return the root string.
      */
      public String getRootString()
      {
@@ -136,6 +195,8 @@ public class ASTReference extends SimpleNode
      *
      *   @param o   unused Object parameter
      *   @param context context used to generate value
+     * @return The execution result.
+     * @throws MethodInvocationException
      */
     public Object execute(Object o, InternalContextAdapter context)
         throws MethodInvocationException
@@ -150,9 +211,10 @@ public class ASTReference extends SimpleNode
 
         Object result = getVariableValue(context, rootString);
 
-        if (result == null)
+        if (result == null && !strictRef)
         {
-            return null;
+            return EventHandlerUtil.invalidGetMethod(rsvc, context, 
+                    "$" + rootString, null, null, uberInfo);
         }
 
         /*
@@ -170,31 +232,75 @@ public class ASTReference extends SimpleNode
 
         try
         {
+            Object previousResult = result; 
+            int failedChild = -1;
             for (int i = 0; i < numChildren; i++)
             {
-                result = jjtGetChild(i).execute(result,context);
-
-                if (result == null)
+                if (strictRef && result == null)
                 {
-                    return null;
+                    /**
+                     * At this point we know that an attempt is about to be made
+                     * to call a method or property on a null value.
+                     */
+                    String name = jjtGetChild(i).getFirstToken().image;
+                    throw new VelocityException("Attempted to access '"  
+                        + name + "' on a null value at "
+                        + Log.formatFileString(uberInfo.getTemplateName(),
+                        + jjtGetChild(i).getLine(), jjtGetChild(i).getColumn()));                  
+                }
+                previousResult = result;
+                result = jjtGetChild(i).execute(result,context);
+                if (result == null && !strictRef)  // If strict and null then well catch this
+                                                   // next time through the loop
+                {
+                    failedChild = i;
+                    break;
                 }
             }
 
+            if (result == null)
+            {
+                if (failedChild == -1)
+                {
+                    result = EventHandlerUtil.invalidGetMethod(rsvc, context, 
+                            "$" + rootString, previousResult, null, uberInfo);                    
+                }
+                else
+                {
+                    StringBuffer name = new StringBuffer("$").append(rootString);
+                    for (int i = 0; i <= failedChild; i++)
+                    {
+                        Node node = jjtGetChild(i);
+                        if (node instanceof ASTMethod)
+                        {
+                            name.append(".").append(((ASTMethod) node).getMethodName()).append("()");
+                        }
+                        else
+                        {
+                            name.append(".").append(node.getFirstToken().image);
+                        }
+                    }
+                    
+                    if (jjtGetChild(failedChild) instanceof ASTMethod)
+                    {
+                        String methodName = ((ASTMethod) jjtGetChild(failedChild)).getMethodName();
+                        result = EventHandlerUtil.invalidMethod(rsvc, context, 
+                                name.toString(), previousResult, methodName, uberInfo);                                                                
+                    }
+                    else
+                    {
+                        String property = jjtGetChild(failedChild).getFirstToken().image;
+                        result = EventHandlerUtil.invalidGetMethod(rsvc, context, 
+                                name.toString(), previousResult, property, uberInfo);                        
+                    }
+                }
+                
+            }
+            
             return result;
         }
         catch(MethodInvocationException mie)
         {
-            /*
-             *  someone tossed their cookies
-             */
-
-            rsvc.error("Method " + mie.getMethodName()
-                        + " threw exception for reference $"
-                        + rootString
-                        + " in template " + context.getCurrentTemplateName()
-                        + " at " +  " [" + this.getLine() + ","
-                        + this.getColumn() + "]");
-
             mie.setReferenceName(rootString);
             throw mie;
         }
@@ -206,100 +312,147 @@ public class ASTReference extends SimpleNode
      *
      *  @param context  context of data to use in getting value
      *  @param writer   writer to render to
+     * @return True if rendering was successful.
+     * @throws IOException
+     * @throws MethodInvocationException
      */
-    public boolean render(InternalContextAdapter context, Writer writer)
-        throws IOException, MethodInvocationException
+    public boolean render(InternalContextAdapter context, Writer writer) throws IOException,
+            MethodInvocationException
     {
-
         if (referenceType == RUNT)
         {
-            writer.write(rootString);
+            if (context.getAllowRendering())
+            {
+                writer.write(rootString);
+            }
+
             return true;
         }
 
         Object value = execute(null, context);
 
+        String localNullString = null;
+
         /*
-         *  if this reference is escaped (\$foo) then we want to do one of two things :
-         *  1) if this is a reference in the context, then we want to print $foo
-         *  2) if not, then \$foo  (its considered shmoo, not VTL)
+         * if this reference is escaped (\$foo) then we want to do one of two things : 1) if this is
+         * a reference in the context, then we want to print $foo 2) if not, then \$foo (its
+         * considered schmoo, not VTL)
          */
 
         if (escaped)
         {
+            localNullString = getNullString(context);
+            
             if (value == null)
             {
-                writer.write(escPrefix);
-                writer.write("\\");
-                writer.write(nullString);
+                if (context.getAllowRendering())
+                {
+                    writer.write(escPrefix);
+                    writer.write("\\");
+                    writer.write(localNullString);
+                }
             }
             else
             {
-                writer.write(escPrefix);
-                writer.write(nullString);
+                if (context.getAllowRendering())
+                {
+                    writer.write(escPrefix);
+                    writer.write(localNullString);
+                }
             }
-
             return true;
         }
 
         /*
-         *  the normal processing
-         *
-         *  if we have an event cartridge, get a new value object
+         * the normal processing
+         * 
+         * if we have an event cartridge, get a new value object
          */
 
-        EventCartridge ec = context.getEventCartridge();
+        value = EventHandlerUtil.referenceInsert(rsvc, context, literal(), value);
 
-        if (ec != null)
+        String toString = null;
+        if (value != null)
         {
-            value =  ec.referenceInsert(literal(), value);
-        }
 
-        /*
-         *  if value is null...
-         */
-
-        if (value == null)
-        {
-            /*
-             *  write prefix twice, because it's shmoo, so the \ don't escape each other...
-             */
-
-            writer.write(escPrefix);
-            writer.write(escPrefix);
-            writer.write(morePrefix);
-            writer.write(nullString);
-
-            if (referenceType != QUIET_REFERENCE
-                && rsvc.getBoolean(RuntimeConstants.RUNTIME_LOG_REFERENCE_LOG_INVALID,
-                        true))
+            if(value instanceof Renderable && ((Renderable)value).render(context,writer))
             {
-               rsvc.warn(new ReferenceException("reference : template = "
-                                + context.getCurrentTemplateName(), this));
+                return true;
             }
 
+            toString = value.toString();
+        }
+
+        if (value == null || toString == null)
+        {
+            /*
+             * write prefix twice, because it's schmoo, so the \ don't escape each other...
+             */
+
+            if (context.getAllowRendering())
+            {
+                localNullString = getNullString(context);
+
+                writer.write(escPrefix);
+                writer.write(escPrefix);
+                writer.write(morePrefix);
+                writer.write(localNullString);
+            }
+
+            if (logOnNull && referenceType != QUIET_REFERENCE && log.isDebugEnabled())
+            {
+                log.debug("Null reference [template '" + getTemplateName()
+                        + "', line " + this.getLine() + ", column " + this.getColumn() + "] : "
+                        + this.literal() + " cannot be resolved.");
+            }
             return true;
         }
         else
         {
             /*
-             *  non-null processing
+             * non-null processing
              */
 
-            writer.write(escPrefix);
-            writer.write(morePrefix);
-            writer.write(value.toString());
+            if (context.getAllowRendering())
+            {
+                writer.write(escPrefix);
+                writer.write(morePrefix);
+                writer.write(toString);
+            }
 
             return true;
         }
     }
 
+    /**
+     * This method helps to implement the "render literal if null" functionality.
+     * 
+     * VelocimacroProxy saves references to macro arguments (AST nodes) so that if we have a macro
+     * #foobar($a $b) then there is key "$a.literal" which points to the literal presentation of the
+     * argument provided to variable $a. If the value of $a is null, we render the string that was
+     * provided as the argument.
+     * 
+     * @param context
+     * @return
+     */
+    private String getNullString(InternalContextAdapter context)
+    {
+        Object callingArgument = context.get(".literal." + nullString);
+
+        if (callingArgument != null)
+            return ((Node) callingArgument).literal();
+        else
+            return nullString;
+    }
+
     /**
      *   Computes boolean value of this reference
      *   Returns the actual value of reference return type
      *   boolean, and 'true' if value is not null
      *
      *   @param context context to compute value with
+     * @return True if evaluation was ok.
+     * @throws MethodInvocationException
      */
     public boolean evaluate(InternalContextAdapter context)
         throws MethodInvocationException
@@ -316,11 +469,24 @@ public class ASTReference extends SimpleNode
                 return true;
             else
                 return false;
-        }
+        }        
         else
-            return true;
+        {
+            try
+            {
+                return value.toString() != null;
+            }
+            catch(Exception e)
+            {
+                throw new VelocityException("Reference evaluation threw an exception at " 
+                    + Log.formatFileString(this), e);
+            }
+        }
     }
 
+    /**
+     * @see org.apache.velocity.runtime.parser.node.SimpleNode#value(org.apache.velocity.context.InternalContextAdapter)
+     */
     public Object value(InternalContextAdapter context)
         throws MethodInvocationException
     {
@@ -336,6 +502,7 @@ public class ASTReference extends SimpleNode
      *  @param context context object containing this reference
      *  @param value Object to set as value
      *  @return true if successful, false otherwise
+     * @throws MethodInvocationException
      */
     public boolean setValue( InternalContextAdapter context, Object value)
       throws MethodInvocationException
@@ -356,8 +523,9 @@ public class ASTReference extends SimpleNode
 
         if (result == null)
         {
-            rsvc.error(new ReferenceException("reference set : template = "
-                    + context.getCurrentTemplateName(), this));
+            String msg = "reference set is not a valid reference at "
+                    + Log.formatFileString(uberInfo);
+            log.error(msg);
             return false;
         }
 
@@ -371,8 +539,18 @@ public class ASTReference extends SimpleNode
 
             if (result == null)
             {
-                rsvc.error(new ReferenceException("reference set : template = "
-                        + context.getCurrentTemplateName(), this));
+                if (strictRef)
+                {
+                    String name = jjtGetChild(i+1).getFirstToken().image;
+                    throw new MethodInvocationException("Attempted to access '"  
+                        + name + "' on a null value", null, name, uberInfo.getTemplateName(),
+                        jjtGetChild(i+1).getLine(), jjtGetChild(i+1).getColumn());
+                }            
+              
+                String msg = "reference set is not a valid reference at "
+                    + Log.formatFileString(uberInfo);
+                log.error(msg);
+
                 return false;
             }
         }
@@ -390,7 +568,18 @@ public class ASTReference extends SimpleNode
                             value, uberInfo);
 
             if (vs == null)
-                return false;
+            {
+                if (strictRef)
+                {
+                    throw new MethodInvocationException("Object '" + result.getClass().getName() +
+                       "' does not contain property '" + identifier + "'", null, identifier,
+                       uberInfo.getTemplateName(), uberInfo.getLine(), uberInfo.getColumn());
+                }
+                else
+                {
+                  return false;
+                }
+            }
 
             vs.invoke(result, value);
         }
@@ -404,18 +593,25 @@ public class ASTReference extends SimpleNode
                 "ASTReference : Invocation of method '"
                 + identifier + "' in  " + result.getClass()
                 + " threw exception "
-                + ite.getTargetException().getClass(),
-               ite.getTargetException(), identifier );
+                + ite.getTargetException().toString(),
+               ite.getTargetException(), identifier, getTemplateName(), this.getLine(), this.getColumn());
+        }
+        /**
+         * pass through application level runtime exceptions
+         */
+        catch( RuntimeException e )
+        {
+            throw e;
         }
         catch(Exception e)
         {
             /*
              *  maybe a security exception?
              */
-            rsvc.error("ASTReference setValue() : exception : " + e
-                                  + " template = " + context.getCurrentTemplateName()
-                                  + " [" + this.getLine() + "," + this.getColumn() + "]");
-            return false;
+            String msg = "ASTReference setValue() : exception : " + e
+                          + " template at " + Log.formatFileString(uberInfo);
+            log.error(msg, e);
+            throw new VelocityException(msg, e);
          }
 
         return true;
@@ -458,7 +654,7 @@ public class ASTReference extends SimpleNode
             if (i == -1)
             {
                 /* yikes! */
-                rsvc.error("ASTReference.getRoot() : internal error : "
+                log.error("ASTReference.getRoot() : internal error : "
                             + "no $ found for slashbang.");
                 computableReference = false;
                 nullString = t.image;
@@ -556,6 +752,7 @@ public class ASTReference extends SimpleNode
          *  we are working with.
          */
 
+        // FIXME: this is the key to render nulls as literals, we need to look at context(refname+".literal") 
         nullString = literal();
 
         if (t.image.startsWith("$!"))
@@ -618,9 +815,38 @@ public class ASTReference extends SimpleNode
 
     }
 
-    public Object getVariableValue(Context context, String variable)
+    /**
+     * @param context
+     * @param variable
+     * @return The evaluated value of the variable.
+     * @throws MethodInvocationException
+     */
+    public Object getVariableValue(Context context, String variable) throws MethodInvocationException
     {
-        return context.get(variable);
+        Object obj = null;
+        try
+        {
+            obj = context.get(variable);
+        }
+        catch(RuntimeException e)
+        {
+            log.error("Exception calling reference $" + variable + " at "
+                      + Log.formatFileString(uberInfo));
+            throw e;
+        }
+        
+        if (strictRef && obj == null)
+        {
+          if (!context.containsKey(variable))
+          {
+              log.error("Variable $" + variable + " has not been set at "
+                        + Log.formatFileString(uberInfo));
+              throw new MethodInvocationException("Variable $" + variable +
+                  " has not been set", null, identifier,
+                  uberInfo.getTemplateName(), uberInfo.getLine(), uberInfo.getColumn());            
+          }
+        }
+        return obj;        
     }
 
 
@@ -651,12 +877,14 @@ public class ASTReference extends SimpleNode
      *  Returns the literal representation of the
      *  node.  Should be something like
      *  $<token>.
+     * @return A literal string.
      */
     public String literal()
     {
         if (literal != null)
             return literal;
-
+        
+        // this value could be cached in this.literal but it increases memory usage
         return super.literal();
     }
 }
