@@ -1,19 +1,22 @@
 package org.apache.velocity.runtime.resource;
 
 /*
- * Copyright 2001,2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.    
  */
 
 import org.apache.velocity.runtime.RuntimeServices;
@@ -31,7 +34,7 @@ import org.apache.velocity.exception.ParseErrorException;
  *
  * @author <a href="mailto:jvanzyl@apache.org">Jason van Zyl</a>
  * @author <a href="mailto:geirm@optonline.net">Geir Magnusson Jr.</a>
- * @version $Id: Resource.java,v 1.12.4.1 2004/03/03 23:23:01 geirm Exp $
+ * @version $Id: Resource.java 685724 2008-08-13 23:12:12Z nbubna $
  */
 public abstract class Resource
 {
@@ -61,7 +64,7 @@ public abstract class Resource
     protected long lastModified = 0;
 
     /**
-     * The next time the file modification time will be checked (in 
+     * The next time the file modification time will be checked (in
      * milliseconds).
      */
     protected long nextCheck = 0;
@@ -76,18 +79,26 @@ public abstract class Resource
      */
     protected String encoding = RuntimeConstants.ENCODING_DEFAULT;
 
-    /** 
-     *  Resource might require ancillary storage of some kind 
+    /**
+     *  Resource might require ancillary storage of some kind
      */
     protected Object data = null;
 
-    /** 
-     *  Default constructor 
+    /**
+     *  Resource type (RESOURCE_TEMPLATE or RESOURCE_CONTENT)
+     */
+    protected int type;
+
+    /**
+     *  Default constructor
      */
     public Resource()
     {
     }
 
+    /**
+     * @param rs
+     */
     public void setRuntimeServices( RuntimeServices rs )
     {
         rsvc = rs;
@@ -105,10 +116,15 @@ public abstract class Resource
      * indicates whether the resource could be read.
      * @exception ResourceNotFoundException Similar in semantics as
      * returning <code>false</code>.
+     * @throws ParseErrorException
+     * @throws Exception
      */
-    public abstract boolean process() 
+    public abstract boolean process()
         throws ResourceNotFoundException, ParseErrorException, Exception;
 
+    /**
+     * @return True if source has been modified.
+     */
     public boolean isSourceModified()
     {
         return resourceLoader.isSourceModified(this);
@@ -116,16 +132,17 @@ public abstract class Resource
 
     /**
      * Set the modification check interval.
-     * @param interval The interval (in seconds).
+     * @param modificationCheckInterval The interval (in seconds).
      */
     public void setModificationCheckInterval(long modificationCheckInterval)
     {
         this.modificationCheckInterval = modificationCheckInterval;
     }
-    
+
     /**
      * Is it time to check to see if the resource
      * source has been updated?
+     * @return True if resource must be checked.
      */
     public boolean requiresChecking()
     {
@@ -133,7 +150,7 @@ public abstract class Resource
          *  short circuit this if modificationCheckInterval == 0
          *  as this means "don't check"
          */
-        
+
         if (modificationCheckInterval <= 0 )
         {
            return false;
@@ -154,27 +171,30 @@ public abstract class Resource
     {
         nextCheck = System.currentTimeMillis() + ( MILLIS_PER_SECOND *  modificationCheckInterval);
     }
-    
+
     /**
      * Set the name of this resource, for example
      * test.vm.
+     * @param name
      */
     public void setName(String name)
     {
         this.name = name;
-    }        
+    }
 
     /**
      * Get the name of this template.
+     * @return The name of this template.
      */
     public String getName()
     {
         return name;
-    }        
+    }
 
     /**
      *  set the encoding of this resource
      *  for example, "ISO-8859-1"
+     * @param encoding
      */
     public void setEncoding( String encoding )
     {
@@ -184,6 +204,7 @@ public abstract class Resource
     /**
      *  get the encoding of this resource
      *  for example, "ISO-8859-1"
+     * @return The encoding of this resource.
      */
     public String getEncoding()
     {
@@ -193,56 +214,80 @@ public abstract class Resource
 
     /**
      * Return the lastModifed time of this
-     * template.
+     * resource.
+     * @return The lastModifed time of this resource.
      */
     public long getLastModified()
     {
         return lastModified;
-    }        
-    
+    }
+
     /**
      * Set the last modified time for this
-     * template.
+     * resource.
+     * @param lastModified
      */
     public void setLastModified(long lastModified)
     {
         this.lastModified = lastModified;
-    }        
+    }
 
     /**
      * Return the template loader that pulled
      * in the template stream
+     * @return The resource loader for this resource.
      */
     public ResourceLoader getResourceLoader()
     {
         return resourceLoader;
     }
-    
+
     /**
      * Set the template loader for this template. Set
      * when the Runtime determines where this template
      * came from the list of possible sources.
+     * @param resourceLoader
      */
     public void setResourceLoader(ResourceLoader resourceLoader)
     {
         this.resourceLoader = resourceLoader;
-    }        
+    }
 
-    /** 
+    /**
      * Set arbitrary data object that might be used
      * by the resource.
+     * @param data
      */
     public void setData(Object data)
     {
         this.data = data;
     }
-    
+
     /**
      * Get arbitrary data object that might be used
      * by the resource.
+     * @return The data object for this resource.
      */
     public Object getData()
     {
         return data;
-    }        
+    }
+    
+    /**
+     * Sets the type of this Resource (RESOURCE_TEMPLATE or RESOURCE_CONTENT)
+     * @since 1.6
+     */
+    public void setType(int type)
+    {
+        this.type = type;
+    }
+    
+    /**
+     * @return type code of the Resource
+     * @since 1.6
+     */
+    public int getType()
+    {
+        return type;
+    }
 }
